/*
 *
 *  Copyright (C) 1996-2001, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Healthcare Information and Communication Systems
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Module:  wlistctn
 *
 *  Author:  Andrew Hewett
 *
 *  Purpose: Service Class Executive (SCE) - Worklist Find Service Class Provider
 *
 *  Last Update:      $Author: joergr $
 *  Update Date:      $Date: 2001/11/28 13:42:32 $
 *  Source File:      $Source: /share/dicom/cvs-depot/dcmtk/wlistctn/apps/scefind.cc,v $
 *  CVS/RCS Revision: $Revision: 1.23 $
 *  Status:           $State: Exp $
 *
 *  CVS/RCS Log at end of file
 *
 */

#include "osconfig.h"    /* make sure OS specific configuration is included first */

#ifdef HAVE_STDLIB_H
#ifndef  _BCB4
/* workaround for bug in Borland C++ Builder 4 */
BEGIN_EXTERN_C
#endif
#include <stdlib.h>
#ifndef  _BCB4
END_EXTERN_C
#endif
#endif

BEGIN_EXTERN_C
#include <stdio.h>
#include <string.h>
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
END_EXTERN_C

#include "dicom.h"
#include "wlistctn.h"
#include "dimse.h"
#include "diutil.h"
#include "sce.h"
#include "wrklstdb.h"
#include "dcdeftag.h"

#include "scefind.h"


typedef struct {
    WorklistDB  *dbHandle;
    WDBStatus   priorStatus;
    DIC_AE      ourAETitle;
} SCE_FindContext;

static OFBool
isaPendingStatus(WDBStatus status)
{
    return ((status == WDB_Pending) || (status == WDB_Pending_Warning));
}

static const char*
status2string(WDBStatus status) 
{
    return DU_cfindStatusString((Uint16)status);
}

static void
addStatusDetail(DcmDataset **statusDetail, const DcmElement *elem)
{
    if (elem == NULL) {
        return;
    }

    if (*statusDetail == NULL) {
        *statusDetail = new DcmDataset();
    }

    DcmVR vr(elem->ident());

    fprintf(stderr, "  Status Detail: "); 

    switch (elem->ident()) {
    case EVR_LO: {
        DcmLongString *lo = new DcmLongString(*((DcmLongString*)elem));
        if (lo->getLength() > vr.getMaxValueLength()) {
            errmsg("addStatusDetail: INTERNAL ERROR: value too large (max %d) for %s: ",
               vr.getMaxValueLength(), vr.getVRName());
        }
        (*statusDetail)->insert(lo, OFTrue /*replaceOld*/);
        lo->print(CERR);
        break;
    }
    case EVR_AT: {
        DcmAttributeTag *at = new DcmAttributeTag(*((DcmAttributeTag*)elem));
        if (at->getLength() > vr.getMaxValueLength()) {
            errmsg("addStatusDetail: INTERNAL ERROR: value too large (max %d) for %s: ",
               vr.getMaxValueLength(), vr.getVRName());
        }
        (*statusDetail)->insert(at, OFTrue /*replaceOld*/);
        at->print(CERR);
        break;
    }
    default: {
        /* other status detail is not supported */
        errmsg("addStatusDetail: unsupported status detail type: %s",
               vr.getVRName());
        break;
    }
    }
}

static void
findCallback(
        void *callbackData,  
        OFBool cancelled, T_DIMSE_C_FindRQ * /*request*/, 
        DcmDataset *requestIdentifiers, int responseCount,
        T_DIMSE_C_FindRSP *response,
        DcmDataset **responseIdentifiers,
        DcmDataset **statusDetail)
    /*
     * This function will try to select another record from a database which matches the search mask that was passed.
     * In certain circumstances, the selected information will be dumped to stdout.
     * 
     * Parameters:
     *   callbackData        - [in] data for this callback function
     *   cancelled           - [in] Specifies if we encounteres a C-CANCEL-RQ. In such a case the search shall be
     *                         cancelled.
     *   request             - [in] The original C-FIND-RQ message.
     *   requestIdentifiers  - [in] Contains the search mask.
     *   responseCount       - [in] If we labelled C-FIND-RSP messages consecutively, starting at label "1", this
     *                         number would provide the label's number.
     *   response            - [inout] the C-FIND-RSP message (will be sent after the call to this function). The status
     *                         field will be set in this function.
     *   responseIdentifiers - [inout] Will in the end contain the next record that matches the search mask (captured
     *                         in requestIdentifiers)
     *   statusDetail        - [inout] This variable can be used to capture detailed information with regard to
     *                         the status information which is captured in the status element (0000,0900) of the
     *                         C-FIND-RSP message.
     */
{
    WDBStatus dbstatus;
    SCE_FindContext *context;
    WorklistDB* db;

    context = (SCE_FindContext*)callbackData;   /* recover context */
    db = context->dbHandle;
    db->setVerbose(opt_verbose);

    /* determine the current db status */
    dbstatus = context->priorStatus;
    
    /* if this is the first time this callback function is called */
    if (responseCount == 1) {
        /* dump some information if required */
        if (opt_verbose) {
            printf("Find SCP Request Identifiers:\n");
            requestIdentifiers->print(COUT);
            printf("=============================\n");
        }
        /* determine the records which match the search mask (after the call, */
        /* these records will be available through db->nextFindResponse(...)) */
        dbstatus = db->startFindRequest(*requestIdentifiers);

        if (!isaPendingStatus(dbstatus) && opt_debug) {
            errmsg(
                "findSCP: Worklist Database: startFindRequest() Failed (%s):",
                status2string(dbstatus));
        }

        if (opt_verbose) {
            printf("=============================\n");
        }
    }

    /* if opt_sleepDuringFind is set the application is supposed */
    /* to sleep n seconds during the finding process */
    if (opt_sleepDuringFind > 0) {
        if (opt_verbose) {
            printf("SLEEPING (during find): %ld secs\n", opt_sleepDuringFind);
        }
        sleep((unsigned int)opt_sleepDuringFind);
    }
    
    /* check if we encountered a C-CANCEL-RQ. The search shall */
    /* additionally only be cancelled if we have pending responses */
    if (cancelled && isaPendingStatus(dbstatus)) {
        dbstatus = db->cancelFindRequest();
    }

    /* if the dbstatus is "pending" try to select another matching record */
    if (isaPendingStatus(dbstatus)) {
        /* create the reponse identifiers */
        *responseIdentifiers = new DcmDataset();

        /* get the next response (record/data set that matches the search mask) */
        *responseIdentifiers = db->nextFindResponse(dbstatus);  
        if (dbstatus != WDB_Success && !isaPendingStatus(dbstatus) && opt_debug) {
            errmsg(
                "findSCP: Worklist Database: nextFindResponse() Failed (%s):",
                status2string(dbstatus));
        }
    }

    /* dump some information if required */
    if (opt_verbose) {
        printf("Worklist Find SCP Response %d [status: %s]\n", responseCount,
            status2string(dbstatus));
        if (*responseIdentifiers != NULL && (*responseIdentifiers)->card() > 0) {
            printf("Response Identifiers (%d)\n", responseCount);
            (*responseIdentifiers)->print(COUT);
            printf("-------\n");
        }
    }

    /* a pending status will cause the response identifiers to be deleted by the calling function. */
    if (!isaPendingStatus(dbstatus) && *responseIdentifiers != NULL) {
        /* logic check */
        if ((*responseIdentifiers)->card() > 0) {
            errmsg("INTERNAL ERROR: non-pending status with a response!");
        }
        /* delete the response identifers */
        delete *responseIdentifiers;
        *responseIdentifiers = NULL;
    }

    /* set response status */
    response->DimseStatus = dbstatus;

    /* delete status detail information if there is some */
    if (*statusDetail != NULL) {
        delete *statusDetail;
        *statusDetail = NULL;
    }

    /* Status detail may only be returned for particular status codes. */
    switch (dbstatus) {
    case WDB_Failed_IdentifierDoesNotMatchSOPClass:
    case WDB_Failed_UnableToProcess:
        addStatusDetail(statusDetail, db->getOffendingElements());
        addStatusDetail(statusDetail, db->getErrorComments());
        break;
    case WDB_Refused_OutOfResorces:
        /* out of resources may only have error comment detail */
        addStatusDetail(statusDetail, db->getErrorComments());
        break;
    default:
        /* other status codes may not have any status detail */
        break;
    }

    /* return */
}


OFCondition
SCE_findSCP(T_ASC_Association * assoc, T_DIMSE_C_FindRQ * request,
        T_ASC_PresentationContextID presID,
        WorklistDB &dbHandle)

{
    SCE_FindContext context;

    context.dbHandle = &dbHandle;
    context.priorStatus = WDB_Pending;
    ASC_getAPTitles(assoc->params, NULL, context.ourAETitle, NULL);

    if (opt_verbose) {
        printf("Received Find SCP: ");
        DIMSE_printCFindRQ(stdout, request);
    }

    OFCondition cond = DIMSE_findProvider(assoc, presID, request, 
        findCallback, &context, DIMSE_BLOCKING, 0);
    if (cond.bad()) {
        errmsg("Find SCP Failed:");
        DimseCondition::dump(cond);
    }

    if (opt_sleepAfterFind > 0) {
        if (opt_verbose) {
            printf("SLEEPING (after find): %ld secs\n", opt_sleepAfterFind);
        }
        sleep((unsigned int)opt_sleepAfterFind);
    }
    return cond; 
}


/*
** CVS Log
** $Log: scefind.cc,v $
** Revision 1.23  2001/11/28 13:42:32  joergr
** Check return value of DcmItem::insert() statements where appropriate to
** avoid memory leaks when insert procedure fails.
**
** Revision 1.22  2001/11/12 16:11:10  meichel
** Removed all ctndisp related code from wlistctn
**
** Revision 1.21  2001/11/06 14:19:55  wilkens
** Added more comments.
**
** Revision 1.20  2001/11/01 14:56:36  wilkens
** Added lots of comments.
**
** Revision 1.19  2001/10/12 12:02:28  meichel
** Adapted wlistctn to OFCondition based dcmnet module (supports strict mode).
**
** Revision 1.18  2001/06/01 15:51:44  meichel
** Updated copyright header
**
** Revision 1.17  2000/04/14 16:40:42  meichel
** Removed default value from output stream passed to print() method.
**   Required for use in multi-thread environments.
**
** Revision 1.16  2000/03/08 16:37:44  meichel
** Updated copyright header.
**
** Revision 1.15  2000/03/03 14:18:22  meichel
** Implemented library support for redirecting error messages into memory
**   instead of printing them to stdout/stderr for GUI applications.
**
** Revision 1.14  2000/02/23 15:13:56  meichel
** Corrected macro for Borland C++ Builder 4 workaround.
**
** Revision 1.13  2000/02/01 11:00:59  meichel
** Avoiding to include <stdlib.h> as extern "C" on Borland C++ Builder 4,
**   workaround for bug in compiler header files.
**
** Revision 1.12  1999/05/03 14:14:45  joergr
** Minor code purifications to keep Sun CC 2.0.1 quiet.
**
** Revision 1.11  1999/04/26 12:59:11  meichel
** Ported Worklist CTN to Win32 environment.
**
** Revision 1.10  1998/08/10 08:57:26  meichel
** renamed member variable in DIMSE structures from "Status" to "DimseStatus".
**
** Revision 1.9  1998/07/24 13:29:01  meichel
** changed WorklistDB::nextFindResponse() parameters
**   in order to avoid the use of an implicit assignment
**   operator for class DcmDataset.
**
** Revision 1.8  1997/07/21 09:08:52  andreas
** - Replace all boolean types (BOOLEAN, CTNBOOLEAN, DICOM_BOOL, BOOL)
**   with one unique boolean type OFBool.
**
** Revision 1.7  1997/05/30 07:33:27  meichel
** Added space characters around comments and simplified
** some inlining code (needed for SunCC 2.0.1).
**
** Revision 1.6  1996/09/27 08:54:26  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C
**
** Revision 1.5  1996/08/29 16:16:08  hewett
** Corrected copyright header.
**
** Revision 1.4  1996/05/01 17:33:49  hewett
** Modified for latest Worklist DB.
**
** Revision 1.3  1996/04/29 19:17:23  hewett
** Updated for existance of WorklistDB::getErrorComments().
**
** Revision 1.2  1996/04/27 13:02:28  hewett
** Now only returns status detail together with status codes which
** are allowed to have status detail (see Supplement 10, Table Z.4-1
** C-FIND Response Status Values).
**
** Revision 1.1.1.1  1996/04/26 14:16:01  hewett
** Initial Import.
**
*/
